"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = connect;
exports.injectStore = void 0;

var _shallowEqual = _interopRequireDefault(require("./utils/shallowEqual"));

var _wrapActionCreators = _interopRequireDefault(require("./utils/wrapActionCreators"));

var _merge = _interopRequireDefault(require("./utils/merge"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultMapStateToProps = function defaultMapStateToProps() {
  return {};
};

var defaultMapDispatchToProps = function defaultMapDispatchToProps(dispatch) {
  return {
    dispatch: dispatch
  };
};

var store;

var injectStore = function injectStore(tinyAppStore) {
  store = tinyAppStore;
};

exports.injectStore = injectStore;

function connect() {
  var mapStateToProps = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultMapStateToProps;
  var mapDispatchToProps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultMapDispatchToProps;
  var finalMapDispatchToProps;

  if (typeof mapDispatchToProps === 'function') {
    finalMapDispatchToProps = mapDispatchToProps;
  } else {
    finalMapDispatchToProps = (0, _wrapActionCreators.default)(mapDispatchToProps);
  }

  return function withConnect(config) {
    var _config$pure = config.pure,
        pure = _config$pure === void 0 ? true : _config$pure;

    function handleStoreChange() {
      if (!this.unsubscribe) {
        return;
      }

      var haveStatePropsChanged = false;
      var haveDispatchPropsChanged = false;
      var currentState = store.getState();
      var nextStateProps = mapStateToProps(currentState, this.props);
      var nextDispatchProps = finalMapDispatchToProps(store.dispatch, this.props);

      if (!pure || !(0, _shallowEqual.default)(nextStateProps, this.stateProps)) {
        haveStatePropsChanged = true;
        this.stateProps = nextStateProps;
      }

      if (!pure || !(0, _shallowEqual.default)(nextDispatchProps, this.dispatchProps)) {
        haveDispatchPropsChanged = true;
        this.dispatchProps = nextDispatchProps;
      }

      if (haveStatePropsChanged || haveDispatchPropsChanged) {
        this.setData(_objectSpread({}, this.stateProps, this.dispatchProps));
      }
    }

    function trySubscribe() {
      this.stateProps = {};
      this.dispatchProps = {};
      this.unsubscribe = store.subscribe(handleStoreChange.bind(this));
      handleStoreChange.call(this);
    }

    function tryUnsubscribe() {
      if (this.unsubscribe) {
        this.unsubscribe();
        this.unsubscribe = null;
      }
    }

    function onLoad() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      trySubscribe.apply(this, args);
    }

    function onUnload() {
      this.stateProps = null;
      this.dispatchProps = null;

      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      tryUnsubscribe.apply(this, args);
    }

    function didMount() {
      for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      trySubscribe.apply(this, args);
    }

    function didUnmount() {
      this.stateProps = null;
      this.dispatchProps = null;

      for (var _len4 = arguments.length, args = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      tryUnsubscribe.apply(this, args);
    }

    var bindActionCreators = _objectSpread({}, finalMapDispatchToProps(store.dispatch), {
      dispatch: store.dispatch
    });

    return (0, _merge.default)(config, _objectSpread({
      onLoad: onLoad,
      onUnload: onUnload,
      didMount: didMount,
      didUnmount: didUnmount
    }, bindActionCreators, {
      // bind action creators on `this`
      methods: bindActionCreators // merge with Component.methods

    }));
  };
}